<?php
/**
 * @package   Automatic Safe Update
 * @author    ABCdatos
 * @license   GPLv2
 * @link      https://taller.abcdatos.net/plugin-actualizar-wordpress/
 */

defined ( 'ABSPATH' ) or die ( 'No se permite el acceso.' );

// Dashboard menu settings
add_action('admin_menu','asu_add_admin_menu');

function asu_add_admin_menu() {
	if ( ! asu_conf_lock_settings () ) {
		if ( asu_errores_config () ){
			$notificacion_contenido = '!';
			$notificacion_globo = " <span class=\"awaiting-mod\">$contenido_notificacion</span>";
		}
		else{
			$notificacion_globo = '';
		}
		add_menu_page (
			__( 'Automatic Safe Update', 'automatic-safe-update' ) . ' - ' . __( 'Settings', 'automatic-safe-update' ), // Page title
			__( 'Automatic Safe Update', 'automatic-safe-update' ) . $notificacion_globo,
			'administrator',                                                      // Capability
			'automatic-safe-update',                                              // Menu slug
			'asu_admin',                                                          // Function
			'data:image/svg+xml;base64,' . base64_encode ( '<svg viewBox="50 150 200 200" xmlns="http://www.w3.org/2000/svg"><path fill="black" d="M232.5938 262.6875 Q232.5938 285.6094 223.3125 305.4375 Q213.6094 326.25 195.75 339.3281 L211.3594 351.5625 L161.1562 365.0625 L165.7969 315.1406 L179.8594 326.25 Q201.6562 306.9844 201.6562 272.6719 Q201.6562 228.6562 159.75 203.9062 L177.4688 169.4531 Q203.3438 181.9688 218.25 208.125 Q232.5938 233.0156 232.5938 262.6875 ZM125.1562 330.6094 L107.4375 365.0625 Q81.8438 352.6875 66.7969 326.3906 Q52.3125 301.2188 52.3125 271.8281 Q52.3125 249.1875 61.7344 229.0781 Q71.4375 208.2656 89.1562 195.1875 L73.5469 182.8125 L123.75 169.4531 L119.1094 219.375 L105.0469 208.2656 Q83.25 227.5312 83.25 261.8438 Q83.25 305.8594 125.1562 330.6094 Z"/></svg>' )         // Icon
		);
	}
}

// Settings page function
function asu_admin() {
	if ( ! current_user_can ( 'manage_options' ) ) {
		wp_die ( 'No tienes suficientes permisos para acceder a esta página.' );
	}
?>
	<div class="wrap">
		<h1><?php echo __( 'Automatic Safe Update Settings' , 'automatic-safe-update' ) . ' <small>v' . asu_get_version (); ?></small></h1>
		<form method="POST" action="options.php">
			<?php
				settings_fields ('automatic-safe-update-ajustes');
				do_settings_sections ( 'automatic-safe-update-ajustes' );
				submit_button();
			?>
		</form>
	</div>

	<div>
	<!--
	<h4>Recomprobando actualizaciones y enviando e-mail si las hay</h4>
	<p>Es probable que no se estén refrescando las listas de actualizaciones en este punto, no sé si sería oportuno o necesario hacerlo. Actualmente creo que solo se hace cuando se aplica el filtro auto_update_plugin, así que la recomprobación manual no estaría afectando, he de ver si meterlo en el pre_set_site_transient_update_plugins cuando haya datos o qué..</p>
	-->
	<?php
			echo '<h3>' . __( 'System status', 'automatic-safe-update' ) . '</h3>';
		
		// Verifica si está activo AUTOMATIC_UPDATER_DISABLED
		if ( defined ( 'AUTOMATIC_UPDATER_DISABLED' ) ) {
			if ( AUTOMATIC_UPDATER_DISABLED ) {
				echo '<h4>AUTOMATIC_UPDATER_DISABLED ';
				echo __( 'is set to' );
				echo ': <b>' . AUTOMATIC_UPDATER_DISABLED . '</b>. ';
				echo __( 'Disable it, typically in wp-config.php if you want to enable plugin updates', 'automatic-safe-update' );
				echo '.</h4>';
			}
		}
		
		// Verifica si está activo WP_AUTO_UPDATE_CORE
		if ( defined ( 'WP_AUTO_UPDATE_CORE' ) ) {
			if ( WP_AUTO_UPDATE_CORE ) {
				echo '<h4>WP_AUTO_UPDATE_CORE ';
				echo __( 'is set to', 'automatic-safe-update' );
				echo ' <b>' . WP_AUTO_UPDATE_CORE . '</b>.</h4>';
			}
		}
		
		asu_anota_log( __FUNCTION__ . " ***** Vaciado del transitorio para forzar comprobación manual *****" );
		set_site_transient( 'update_plugins', null );
		asu_anota_log( __FUNCTION__ . " Llamada manual a recomprobación, con envío final de email." );
		wp_update_plugins();
		asu_anota_log( __FUNCTION__ . " Carga de lista de actualizaciones" );
		
		$cantidad_pendientes = asu_cantidad_actualizaciones_pendientes ();
		
		if ( !$cantidad_pendientes ) {
			asu_anota_log( __FUNCTION__ . " Todos los plugins están actualizados" );
			echo '<h4>' . __( 'All plugins are up to date.', 'automatic-safe-update' ) . '</h4>';
		}
		else {
			asu_anota_log( __FUNCTION__ . ' Hay ' . mensaje_pendientes_actualizar ( $cantidad_pendientes ) );
			echo '<h4>' . mensaje_pendientes_actualizar ( $cantidad_pendientes ) . '.</h4>';
			asu_envia_email_actualizaciones ();
		}
		
		//echo str_replace ( "\n", "<br />\n", esc_html ( asu_reprocesa_lista_compacta ( asu_lista_actualizaciones_pendientes () , true ) ) );
		asu_imprime_lista_compacta ( asu_reprocesa_lista_compacta ( asu_lista_actualizaciones_pendientes () , true ) );

		echo '<h3>' . __( 'Log', 'automatic-safe-update' ) . '</h3>';
		asu_imprime_log ();
		
	?>
	</div>
<?php
}



// Settings page functionality

add_action ( 'admin_init', 'asu_settings_init' );
function asu_settings_init () {
	foreach ( asu_lista_opciones() as $nombre_opcion ) {
		if ( 'asu_actualizacion_omision' === $nombre_opcion ) {
			register_setting ( 'automatic-safe-update-ajustes', $nombre_opcion, array ( 'sanitize_callback' => 'asu_sanear_omision', 'type' => 'integer', ) );
		}
		elseif ( 'asu_plugins' === $nombre_opcion ) {
			register_setting ( 'automatic-safe-update-ajustes', $nombre_opcion, 'asu_sanear_plugins' );
		}
		else{
			register_setting ( 'automatic-safe-update-ajustes', $nombre_opcion );
		}
	}

	add_settings_section (
		'asu_seccion_general',                           // $id (string) (Required) Slug-name to identify the section. Used in the 'id' attribute of tags.
		__( 'Basic settings', 'automatic-safe-update' ), // $title (string) (Required) Formatted title of the section. Shown as the heading for the section.
		'asu_seccion_general_callback',                  // $callback (callable) (Required) Function that echos out any content at the top of the section (between heading and fields).
		'automatic-safe-update-ajustes'                  // $page (string) (Required) The slug-name of the settings page on which to show the section. Built-in pages include 'general', 'reading', 'writing', 'discussion', 'media', etc. Create your own using add_options_page();
	);

	add_settings_field (
		'asu_actualizacion_omision',                                     // $id (string) (Required) Slug-name to identify the field. Used in the 'id' attribute of tags.
		__( 'Defaut plugin update level', 'automatic-safe-update' ),     // $title (string) (Required) Formatted title of the field. Shown as the label for the field during output.
		'asu_actualizacion_omision_callback',                            // $callback (callable) (Required) Function that fills the field with the desired form inputs. The function should echo its output.
		'automatic-safe-update-ajustes',                                 // $page (string) (Required) The slug-name of the settings page on which to show the section (general, reading, writing, ...).
		'asu_seccion_general',                                           // $section (string) (Optional) The slug-name of the section of the settings page in which to show the box.
		array ()                                                         // $args (array) (Optional) Extra arguments used when outputting the field.
	);

	add_settings_field (
		'asu_admin_email',                                               // $id (string) (Required) Slug-name to identify the field. Used in the 'id' attribute of tags.
		__( 'Send updates to e-mail address', 'automatic-safe-update' ), // $title (string) (Required) Formatted title of the field. Shown as the label for the field during output.
		'asu_admin_email_callback',                                      // $callback (callable) (Required) Function that fills the field with the desired form inputs. The function should echo its output.
		'automatic-safe-update-ajustes',                                 // $page (string) (Required) The slug-name of the settings page on which to show the section (general, reading, writing, ...).
		'asu_seccion_general',                                           // $section (string) (Optional) The slug-name of the section of the settings page in which to show the box.
		array ()                                                         // $args (array) (Optional) Extra arguments used when outputting the field.
	);

	add_settings_field (
		'asu_lineas_log',
		__( 'Log max lines', 'automatic-safe-update' ),
		'asu_lineas_log_callback',
		'automatic-safe-update-ajustes',
		'asu_seccion_general',
		array ()
	);

	add_settings_field (
		"asu_plugins",
		__( 'Individual plugin setting', 'automatic-safe-update' ),
		'asu_plugins_individuales_callback',
		'automatic-safe-update-ajustes',
		'asu_seccion_general',
		array ()
	);
/*
	add_settings_section (
		'asu_seccion_avanzada',
		__( 'Advanced settings', 'automatic-safe-update' ),
		'asu_seccion_avanzada_callback',
		'automatic-safe-update-ajustes'
	);

	add_settings_field (
		'asu_lock_settings',
		__( 'Lock this settings', 'automatic-safe-update' ),
		'asu_lock_settings_callback',
		'automatic-safe-update-ajustes',
		'asu_seccion_avanzada',
		array ()
	);
*/

	add_settings_section (
		'asu_seccion_experimental',
		__( 'Experimental settings', 'automatic-safe-update' ),
		'asu_seccion_experimental_callback',
		'automatic-safe-update-ajustes'
	);

	add_settings_field (
		'asu_actualizacion_traducciones',
		__( 'Update translations', 'automatic-safe-update' ),
		'asu_actualizacion_traducciones_callback',
		'automatic-safe-update-ajustes',
		'asu_seccion_experimental',
		array ()
	);

	add_settings_field (
		'asu_actualizacion_temas',
		__( 'Update themes', 'automatic-safe-update' ),
		'asu_actualizacion_temas_callback',
		'automatic-safe-update-ajustes',
		'asu_seccion_experimental',
		array ()
	);
/*
*/

/*
	add_settings_field (
		'asu_',
		__( '', 'automatic-safe-update' ),
		'asu__callback',
		'automatic-safe-update-ajustes',
		'asu_seccion_general',
		array ()
	);
*/

}

// Callbacks to show options data
// Callbacks para la presentación de datos de opciones

function asu_seccion_general_callback () { 
	//echo __( 'Email communications data.', 'automatic-safe-update' );
}

function asu_version_callback () {
	// Hidden field to save version number too.
	echo '<input name="asu_version" type="hidden" id="asu_version" value="' . esc_attr( asu_get_version() ) . '" />';
}

function asu_actualizacion_omision_callback () {
	echo plugin_linea_configuracion_omision ( asu_conf_actualizacion_omision () );
	//echo '<input name="asu_actualizacion_omision" type="text" id="asu_actualizacion_omision" value="' . esc_attr( asu_conf_actualizacion_omision () ) . '" class="regular-text" />';
	echo '<p class="description" id="tagline-description">' . __( 'Default max plugin version level to update.', 'automatic-safe-update' ) . '</p>';
}

function asu_admin_email_callback () {
	echo '<input name="asu_admin_email" type="text" id="asu_admin_email" value="' . esc_attr( asu_conf_admin_email () ) . '" class="regular-text" />';
	//Dirección a la que se notificarán las actualizaciones pendientes y aplicadas.
	echo '<p class="description" id="tagline-description">' . __( 'Notification address for pending and applied updates.', 'automatic-safe-update' ) . '</p>';
}

function asu_lineas_log_callback () {
	echo '<input name="asu_lineas_log" type="text" id="asu_lineas_log" value="' . esc_attr( asu_conf_lineas_log () ) . '" class="regular-text" />';
	echo '<p class="description" id="tagline-description">' . __( 'Maximum lines amount to keep in log.', 'automatic-safe-update' ) . '</p>';
}

function asu_seccion_avanzada_callback () { 
	//echo __( 'Email communications data.', 'automatic-safe-update' );
}

function asu_lock_settings_callback () {
	echo "<input type='checkbox' name='asu_lock_settings' ";
	checked( asu_conf_lock_settings () , 1 );
	echo " value='1'> ";
	echo __( 'Lock access to this settings page.', 'automatic-safe-update' );
	echo '<p class="description" id="tagline-description" style="color:red;">' . __( 'Don\'t do it if you aren\'t fully sure what are you doing.', 'automatic-safe-update' ) . ' ' . __( 'To restore it, you will need remove or set the value to zero (0) the option asu_lock_settings in options table from database.', 'automatic-safe-update' ) . '</p>';
}

function asu_seccion_experimental_callback () { 
	echo __( 'These settings are only partially supportted and may experience unexpected results.', 'automatic-safe-update' );
}

function asu_actualizacion_traducciones_callback () {
	echo "<input type='checkbox' name='asu_actualizacion_traducciones' ";
	checked( asu_conf_actualizacion_traducciones () , 1 );
	echo " value='1'> ";
	echo __( 'Update translations', 'automatic-safe-update' ) . '.';
	echo '<p class="description" id="tagline-description">' . __( 'Update all translations.', 'automatic-safe-update' ) . '</p>';
}

function asu_actualizacion_temas_callback () {
	echo "<input type='checkbox' name='asu_actualizacion_temas' ";
	checked( asu_conf_actualizacion_temas () , 1 );
	echo " value='1'> ";
	echo __( 'Update themes', 'automatic-safe-update' ) . '.';
	echo '<p class="description" id="tagline-description">' . __( 'Update all themes.', 'automatic-safe-update' ) . '</p>';
}

/*
function asu_plugin_callback ( $args ) {
	$slug  = $args[0];
	$valor = $args[1];
	echo plugin_linea_configuracion ( $slug, $valor );
}
*/

function asu_plugins_individuales_callback () {
	// Hay que recorrer los plugins configurados y los instalados NO configurados
	$lista_plugins = asu_plugins_configurados ();

	// Agrega los plugins instalados no configurados a la lista
	if ( !function_exists ( 'get_plugins' ) || !function_exists ( 'is_plugin_active' ) ) {
		include_once ABSPATH . 'wp-admin/includes/plugin.php';
	}
	$plugins_instalados = get_plugins();
	foreach ( $plugins_instalados as $plugin_file => $plugin_data ) {
		$slug = asu_plugin_slug ( $plugin_file );
		if ( ! plugin_esta_configurado ( $slug ) ) {
			$lista_plugins[ $slug ] = '';
		}
	}

	// Ordena por clave (slug) mejor sería ordenar por tìtulo
	ksort ( $lista_plugins );

	// Campos del formulario para cada plugin
	foreach ( $lista_plugins as $slug => $valor ) {
		echo plugin_linea_configuracion ( $slug, $valor );
	}
}


/*
function asu__callback () {
	echo '<input name="asu_" type="text" id="asu_" value="' . esc_attr( asu_conf_ () ) . '" class="regular-text" />';
	echo '<p class="description" id="tagline-description">' . __( '*************.', 'automatic-safe-update' ) . '</p>';
}
*/

# Para indicar advertencia en la burbuja del menú
function asu_errores_config () {
	return false;
}

function plugin_linea_configuracion ( $slug, $valor ) {
	$html = plugin_html_select_configuracion ( 'asu_plugins['.$slug.']', $valor, true );
	$html.= asu_plugin_nombre ( $slug );
	$html.= "<br />\n";
	return $html;
}

function plugin_linea_configuracion_omision ( $valor ) {
	return plugin_html_select_configuracion ( 'asu_actualizacion_omision', $valor, false );
}

function plugin_html_select_configuracion ( $nombre_campo, $valor, $mostrar_omision = NULL ) {
	$html =     "\t<select name='$nombre_campo'>\n";
	$html.=     "\t<option value='3'" . selected ( $valor, 3 , false ) . ">" . __( 'Major'   , 'automatic-safe-update' ) . "</option>\n";
	$html.=     "\t<option value='2'" . selected ( $valor, 2 , false ) . ">" . __( 'Minor'   , 'automatic-safe-update' ) . "</option>\n";
	$html.=     "\t<option value='1'" . selected ( $valor, 1 , false ) . ">" . __( 'Patch'   , 'automatic-safe-update' ) . "</option>\n";
	$html.=     "\t<option value='0'" . selected ( $valor, 0 , false ) . ">" . __( 'Hold'    , 'automatic-safe-update' ) . "</option>\n";
	if ( $mostrar_omision ) {
		$html.= "\t<option value=''"  . selected ( $valor, '', false ) . ">" . __( 'Default' , 'automatic-safe-update' ) . "</option>\n";
	}
	$html.= "</select>\n";
	return $html;
}

// Elimina valores vacíos para los plugins con configuración por defecto. Así podríamos liquidar plugins no existentes
function asu_sanear_plugins ( $plugins ) {
	foreach ( $plugins as $slug => $valor ) {
		if ( '' === $valor ) {
			unset ( $plugins[ $slug ] );
		}
		else{
			
		}
	}
	return $plugins;
}

function asu_sanear_omision ($nivel) {
	if ( $nivel != 0 && $nivel != 1 && $nivel != 2 && $nivel != 3 ) {
		$nivel = asu_conf_actualizacion_omision ();
	}
	return $nivel;
}

// Settings page function
// Se utiliza para mostrar online los plugins pendientes
function asu_imprime_lista_compacta ( $lista_compacta ) {
	//echo str_replace ( "\n", "<br />\n", esc_html ( asu_reprocesa_lista_compacta ( asu_lista_actualizaciones_pendientes () , true ) ) );
	echo str_replace ( "\n", "<br />\n", esc_html ( $lista_compacta ) );
}

function asu_imprime_log ( ) {
	$log_full = get_option ( asu_log_option_name () );
	echo str_replace ( "\n", "<br />\n", esc_html ( $log_full ) );
}
